import gconf
import os
import gobject


GCONF_PATH = "/apps/traffic_counter"

class Gconf(gobject.GObject):
	__gsignals__ = {
		#@param: self key value
		'property-changed' : (gobject.SIGNAL_NO_HOOKS, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT,)),
	}
	def __init__(self, section=""):
		gobject.GObject.__init__(self)
		self.section = section
		if self.section != "":
			self.section = "/" + self.section
		
		self.client = gconf.client_get_default ()
		
		self.client.add_dir(GCONF_PATH + self.section, gconf.CLIENT_PRELOAD_ONELEVEL)
		self.client.connect("value_changed", self.value_changed)

	# convert a GConfValue to python native value
	def gval2pyval(self, val):
		if val.type == gconf.VALUE_STRING:
			return val.get_string()
		elif val.type == gconf.VALUE_FLOAT:
			return val.get_float()
		elif val.type == gconf.VALUE_INT:
			return val.get_int()
		elif val.type == gconf.VALUE_BOOL:
			return val.get_bool()
		elif val.type == gconf.VALUE_LIST:
			x = []
			for item in val.get_list():
				x.append(self.gval2pyval(item))
			return x
		else:
			raise AttributeError("Cant get this type from gconf")
	
	def value_changed(self, client, key, value):
		if os.path.dirname(key) == GCONF_PATH + self.section:
			name = os.path.basename(key)
			self.emit("property-changed", name, self.get(name))		
	
	def set(self, key, value):
		func = None
	
		if type(value) == str:
			func = self.client.set_string
		elif type(value) == int or type(value) == long:
			func = self.client.set_int
		elif type(value) == bool:
			func = self.client.set_bool
		elif type(value) == float:
			func = self.client.set_float
		elif type(value) == list:
			def x(key, val):
				self.client.set_list(key, gconf.VALUE_STRING, val)
			func = x
		else:
			raise AttributeError("Cant set this type in gconf")
		
		func(GCONF_PATH + self.section + "/" + key, value)
		
	def get(self, key):
		val = self.client.get(GCONF_PATH + self.section + "/" + key)
		if val != None:
			return self.gval2pyval(val)
		else:
			return None
